class BadRequestException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'BadRequestException: {self.status_code} - {self.message}'


class UnauthorizedException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'UnauthorizedException: {self.status_code} - {self.message}'


class ConflictException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'ConflictException: {self.status_code} - {self.message}'


class InternalServerErrorException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'InternalServerErrorException: {self.status_code} - {self.message}'


class BadGatewayException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'BadGatewayException: {self.status_code} - {self.message}'


class RedirectionException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'RedirectionException: {self.status_code} - {self.message}'


class NotFoundException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'NotFoundException: {self.status_code} - {self.message}'


class ForbiddenException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'ForbiddenException: {self.status_code} - {self.message}'


class TooManyRequestsException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'TooManyRequestsException: {self.status_code} - {self.message}'


class ClientErrorException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'ClientErrorException: {self.status_code} - {self.message}'


class ServerErrorException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'ServerErrorException: {self.status_code} - {self.message}'


class UnknownException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'UnknownException: {self.status_code} - {self.message}'


class TimeoutException(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return f'TimeoutException: {self.message}'


class ConnectionErrorException(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return f'ConnectionErrorException: {self.message}'


class NoInternetException(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return f'NoInternetException: {self.message}'


class ServiceUnavailableException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'ServiceUnavailableException: {self.status_code} - {self.message}'


class GatewayTimeoutException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return f'GatewayTimeoutException: {self.status_code} - {self.message}'



import unittest
from unittest import mock

from nanorequests.nanoexceptions import (
    RedirectionException,
    NotFoundException,
    ForbiddenException,
    TooManyRequestsException,
    BadRequestException,
    InternalServerErrorException
)
from nanorequests.nanorequests import NanoRequests


def mock_requests_get(response_status_code, response_json=None, response_text=None):
    mock_response = mock.MagicMock()
    mock_response.status_code = response_status_code
    mock_response.headers.get.return_value = 'application/json'

    if response_json is not None:
        mock_response.json.return_value = response_json

    if response_text is not None:
        mock_response.json.side_effect = ValueError
        mock_response.text = response_text

    return mock_response


class NanoRequestsTests(unittest.TestCase):
    @staticmethod
    def mock_get(response_status_code, response_json=None, response_text=None):
        return mock.patch('nanorequests.nanorequests.requests.get',
                          side_effect=lambda url, **kwargs: mock_requests_get(response_status_code, response_json,
                                                                              response_text))

    def test_get_success_json(self):
        url = "https://api.example.com/users/1"
        expected_response = {'id': 1, 'name': 'John Doe'}

        with self.mock_get(200, response_json=expected_response):
            response = NanoRequests.get(url)
            self.assertEqual(response, expected_response)

    def test_get_success_plain_text(self):
        url = "https://api.example.com/users/2"
        expected_response = {'message': 'User not found'}

        with self.mock_get(200, response_json=expected_response, response_text="User not found"):
            response = NanoRequests.get(url)
            self.assertEqual(response, expected_response)

    def test_get_redirection(self):
        url = "https://api.example.com/users/3"

        with self.mock_get(302):
            with self.assertRaises(RedirectionException):
                NanoRequests.get(url)

    def test_get_not_found(self):
        url = "https://api.example.com/users/4"

        with self.mock_get(404):
            with self.assertRaises(NotFoundException):
                NanoRequests.get(url)

    def test_get_forbidden(self):
        url = "https://api.example.com/users/5"

        with self.mock_get(403):
            with self.assertRaises(ForbiddenException):
                NanoRequests.get(url)

    def test_get_too_many_requests(self):
        url = "https://api.example.com/users/6"

        with self.mock_get(429):
            with self.assertRaises(TooManyRequestsException):
                NanoRequests.get(url)

    def test_get_client_error(self):
        url = "https://api.example.com/users/7"

        with self.mock_get(400):
            with self.assertRaises(BadRequestException):
                NanoRequests.get(url)

    def test_get_server_error(self):
        url = "https://api.example.com/users/8"

        with self.mock_get(500):
            with self.assertRaises(InternalServerErrorException):
                NanoRequests.get(url)


if __name__ == '__main__':
    unittest.main()



import requests

from nanorequests.nanoexceptions import *


class NanoRequests:
    @staticmethod
    def _make_request(method, url, headers=None, json=None, timeout=None, **kwargs):
        try:
            response = method(url, headers=headers, json=json, timeout=timeout, **kwargs)
            return NanoRequests._handle_response(response)
        except requests.Timeout:
            raise TimeoutException('Request timed out.')
        except requests.ConnectionError:
            raise ConnectionErrorException('Connection error.')
        except requests.exceptions.RequestException:
            raise NoInternetException('No internet connection.')

    @staticmethod
    def get(url, headers=None, params=None, timeout=None, **kwargs):
        return NanoRequests._make_request(requests.get, url, headers=headers, params=params, timeout=timeout, **kwargs)

    @staticmethod
    def post(url, headers=None, json=None, timeout=None, **kwargs):
        return NanoRequests._make_request(requests.post, url, headers=headers, json=json, timeout=timeout, **kwargs)

    @staticmethod
    def put(url, headers=None, json=None, timeout=None, **kwargs):
        return NanoRequests._make_request(requests.put, url, headers=headers, json=json, timeout=timeout, **kwargs)

    @staticmethod
    def delete(url, headers=None, timeout=None, **kwargs):
        return NanoRequests._make_request(requests.delete, url, headers=headers, timeout=timeout, **kwargs)

    @staticmethod
    def patch(url, headers=None, json=None, timeout=None, **kwargs):
        return NanoRequests._make_request(requests.patch, url, headers=headers, json=json, timeout=timeout, **kwargs)

    @staticmethod
    def _handle_response(response):
        if response.status_code // 100 == 2:  # 2XX status codes
            if 'application/json' in response.headers.get('content-type', ''):
                try:
                    return response.json()
                except ValueError:
                    return {'message': response.text}
            else:
                return {'message': response.text}
        elif response.status_code // 100 == 3:  # 3XX status codes
            raise RedirectionException(response.status_code, response.text)
        elif response.status_code == 400:  # 400 Bad Request
            raise BadRequestException(response.status_code, response.text)
        elif response.status_code == 401:  # 401 Unauthorized
            raise UnauthorizedException(response.status_code, response.text)
        elif response.status_code == 403:  # 403 Forbidden
            raise ForbiddenException(response.status_code, response.text)
        elif response.status_code == 404:  # 404 Not Found
            raise NotFoundException(response.status_code, response.text)
        elif response.status_code == 409:  # 409 Conflict
            raise ConflictException(response.status_code, response.text)
        elif response.status_code == 429:  # 429 Too Many Requests
            raise TooManyRequestsException(response.status_code, response.text)
        elif response.status_code // 100 == 4:  # 4XX status codes
            raise ClientErrorException(response.status_code, response.text)
        elif response.status_code == 500:  # 500 Internal Server Error
            raise InternalServerErrorException(response.status_code, response.text)
        elif response.status_code == 502:  # 502 Bad Gateway
            raise BadGatewayException(response.status_code, response.text)
        elif response.status_code == 503:  # 503 Service Unavailable
            raise ServiceUnavailableException(response.status_code, response.text)
        elif response.status_code == 504:  # 504 Gateway Timeout
            raise GatewayTimeoutException(response.status_code, response.text)
        else:
            raise UnknownException(response.status_code, response.text)



from setuptools import setup, find_packages


setup(
    name='nanorequests',
    version='0.9',
    license='MIT',
    author="Viral Parmar",
    description='The simplest REST requests package for quick API integration',
    author_email='viralparmarme@gmail.com',
    packages=['nanorequests'],
    url='https://github.com/viralparmarme/nanorequests',
    keywords='nanorequests viralparmarme viralparmar',
    py_modules=['nanorequests', 'exceptions'],
    long_description="README.md",
    long_description_content_type="text/markdown",
    install_requires=[
        'requests'
    ],
)
